__all__ = ['EtcdLock']

import asyncio

import aetcd


class EtcdLock:
    '''
    A distributed lock.
    '''
    host: str
    port: int
    username: str | None
    password: str | None
    timeout: int | None
    client: aetcd.Client
    lock: aetcd.Lock


    def __init__(self,
                 host: str,
                 port: int,
                 name: str,
                 username: str | None = None,
                 password: str | None = None,
                 timeout: int | None = None,
    ):
        self.host = host
        self.port = port
        self.name = name
        self.username = username
        self.password = password
        self.timeout = timeout
        self.client = aetcd.Client(host=host, port=port, username=username, password=password, timeout=timeout)
        self.lock = self.client.lock(name)


    async def __aenter__(self):
        await self.lock.acquire()
        return self


    async def __aexit__(self, exc_type, exc_value, exc_tb):
        if exc_type:
            await self.close()
            return self

        await self.lock.release()
        return self


    async def acquire(self, timeout: float = 10.0) -> bool:
        '''
        Acquire the lock.
        '''
        r: bool = await self.lock.acquire()
        return r


    async def release(self) -> bool:
        '''
        Release the lock.
        '''
        r: bool = await self.lock.release()
        return r


    async def close(self):
        '''
        Close the lock.
        '''
        await self.client.close()


    async def is_acquired(self) -> bool:
        '''
        Check if this lock is currently acquired.
        '''
        is_acquired: bool = await self.lock.is_acquired()
        return is_acquired


"""
async def main():
    HOST = 'etcd-test'
    PORT = 2379

    # lock0
    lock0 = EtcdLock(host=HOST, port=PORT, name='lock-0')
    await lock0.acquire()
    await asyncio.sleep(10.0)
    await lock0.release()
    await lock0.close()

    # lock1
    lock1 = EtcdLock(host=HOST, port=PORT, name='lock-1')
    
    async with lock1:
        await asyncio.sleep(10.0)
    
    await lock1.close()


if __name__ == '__main__':
    asyncio.run(main())
"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['thlock']

package_data = \
{'': ['*']}

install_requires = \
['aetcd3>=0.1.0a7,<0.2.0']

setup_kwargs = {
    'name': 'thlock',
    'version': '0.9.2',
    'description': 'TangledHub thlock library',
    'long_description': "[![Build][build-image]]()\n[![Status][status-image]][pypi-project-url]\n[![Stable Version][stable-ver-image]][pypi-project-url]\n[![Coverage][coverage-image]]()\n[![Python][python-ver-image]][pypi-project-url]\n[![License][bsd3-image]][bsd3-url]\n\n\n# thlock\n\n## Overview\nTangledHub library for etcd_lock with a focus on asynchronous functions\n\n## Licencing\nthlock is licensed under the BSD license. Check the [LICENSE](https://opensource.org/licenses/BSD-3-Clause) for details\n\n---\n\n## Installation\n```bash\npip install thlock\n```\n\n## Testing\n```bash\ndocker-compose build thlock-test ; docker-compose run --rm thlock-test\n```\n\n## Building\n```bash\ndocker-compose build thlock-build ; docker-compose run --rm thlock-build\n```\n\n## Publish\n```bash\ndocker-compose build thcrypto-lock ; docker-compose run --rm -e PYPI_USERNAME=__token__ -e PYPI_PASSWORD=__SECRET__ thlock-publish\n```\n\n---\n\n## Usage\n\n### setup\n\nCreate instance of EtcdLock\n\n```python\n\nHOST = 'etcd-test'\nPORT = 2379\n\n# create instance of EtcdLock\nlock = EtcdLock(host=HOST, port=PORT, name='lock-0')\n\n```\n\n\n### Acquire lock\n\n```python\n\nHOST = 'etcd-test'\nPORT = 2379\n\n# create instance of EtcdLock\nlock = EtcdLock(host=HOST, port=PORT, name='lock-0')\n\n# acquire lock\nawait lock.acquire()\n\n```\n\n\n<!-- Links -->\n\n<!-- Badges -->\n[bsd3-image]: https://img.shields.io/badge/License-BSD_3--Clause-blue.svg\n[bsd3-url]: https://opensource.org/licenses/BSD-3-Clause\n[build-image]: https://img.shields.io/badge/build-success-brightgreen\n[coverage-image]: https://img.shields.io/badge/Coverage-100%25-green\n\n[pypi-project-url]: https://pypi.org/project/thlock/\n[stable-ver-image]: https://img.shields.io/pypi/v/thlock?label=stable\n[python-ver-image]: https://img.shields.io/pypi/pyversions/thlock.svg?logo=python&logoColor=FBE072\n[status-image]: https://img.shields.io/pypi/status/thlock.svg\n\n\n",
    'author': 'TangledHub',
    'author_email': 'info@tangledgroup.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://gitlab.com/tangledlabs/thlock',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
